from typing import Annotated
from paxpar.cli.command.version import version_get
import typer
from rich.console import Console
from paxpar.cli.tools import PaxparCLI_ObjCtx, call, root_command_callback


console = Console()

app = typer.Typer(
    help="pp deploy commands",
    callback=root_command_callback(),
)

"""
❯ scw container container list
ID                                    NAME                      NAMESPACE ID                          STATUS  MIN SCALE  MAX SCALE  MEMORY LIMIT  CPU LIMIT  TIMEOUT    ERROR MESSAGE  PRIVACY
6b0e651b-cc7e-4255-9264-013d1ebb8066  proxy                     8080c26f-515c-40c7-b388-fb168520207c  ready   0          1          2048          1120       5 minutes  -              public
36a7a4e1-dd3e-47d1-b381-ae003518b832  container-stoic-herschel  8080c26f-515c-40c7-b388-fb168520207c  ready   0          5          2048          1120       5 minutes  -              public
5a6669b7-cfd2-4076-809e-8a5dd14d7bea  gotenberg6-phentz         8080c26f-515c-40c7-b388-fb168520207c  ready   0          5          2048          1120       5 minutes  -              public
9645138c-a20a-4f4f-ac9b-be79a2d9f01f  gotenberg8                8080c26f-515c-40c7-b388-fb168520207c  ready   0          5          2048          1120       5 minutes  -              public
f8801dd4-a65b-44d6-b888-9aba928cad2e  simple-web                8080c26f-515c-40c7-b388-fb168520207c  ready   0          5          256           250        5 minutes  -              public
30a89147-362d-44aa-a961-7f3a158dbab9  paxpar-core-2c48f2d6      8080c26f-515c-40c7-b388-fb168520207c  ready   0          5          3072          1000       5 minutes  -              public
"""


@app.command(name="list")
def list_deployments(
    ctx: typer.Context,
):
    """
    List deployed instance
    """
    # TODO: get JSON with "-o"
    call("""scw container container list""")


@app.command()
def api(
    ctx: typer.Context,
    target: str = "arundo_cluster",
    version: Annotated[str | None, typer.Argument()] = None,
    conf_file: list[str] = [],
    conf_file1: Annotated[str | None, typer.Option(envvar="PP_CONF_1")] = None,
    conf_file2: Annotated[str | None, typer.Option(envvar="PP_CONF_2")] = None,
    conf_file3: Annotated[str | None, typer.Option(envvar="PP_CONF_2")] = None,
    helm_values_file: list[str] = [],
    helm_values: list[str] = [],
    gitlab_token_registry_read: Annotated[
        str, typer.Option(envvar="GITLAB_TOKEN_REGISTRY_READ")
    ] = "xxx",
    gitea_token_registry_read: Annotated[
        str, typer.Option(envvar="GITEA_TOKEN_REGISTRY_READ")
    ] = "xxx",
    # pax-partitus project id
    deploy_gitlab_project_id: Annotated[
        str, typer.Option(envvar="DEPLOY_GITLAB_PROJECT_ID")
    ] = "8545789",
    # DEPLOY_VERSION_RELEASE: "d4276905"
    deploy_helm_chart_name: Annotated[
        str, typer.Option(envvar="DEPLOY_HELM_CHART_NAME")
    ] = "paxpar",
    # the namespace to deploy to
    deploy_k8s_namespace: Annotated[
        str, typer.Option(envvar="DEPLOY_K8S_NAMESPACE")
    ] = "paxpar-dev",
    # cluster agent to deploy to
    # cf https://gitlab.com/arundo-tech/paxpar-gitops/-/clusters
    gitlab_k8s_agent: Annotated[
        str, typer.Option(envvar="GITLAB_K8S_AGENT")
    ] = "arundo-cluster2024",

    ci_job_token: Annotated[
        str, typer.Option(envvar="CI_JOB_TOKEN")
    ] = "xxx",

    
):
    """
    Deploy paxpar API
    """
    ctx_obj: PaxparCLI_ObjCtx = ctx.obj

    assert target == "arundo_cluster"

    # version = "4.2.37"
    if version is None:
        version = version_get(ctx_obj)
        if ctx_obj.verbose:
            print(f"No version specified, assuming {version}")

    # print("conf_file", conf_file)

    # from infra/paxpar-front-builder/back-release.yaml

    # only needed for test
    # DEPLOY_INGRESS_HOST = "api.dev.paxpar.io"

    # set kubernetes context to the right cluster
    # example:  kubectl config use-context arundo-tech/paxpar-gitops:paxpar-clust1
    call(f"""
         kubectl config use-context arundo-tech/paxpar-gitops:{gitlab_k8s_agent}
    """,
        ctx_obj=ctx_obj,)

    # TODO: also add issuers ressources here ?
    # add the gitlab helm repo
    call(
        f"""
        helm repo add \
            --username cicd \
            --password {ci_job_token} \
            repo{deploy_gitlab_project_id} \
            https://gitlab.com/api/v4/projects/{deploy_gitlab_project_id}/packages/helm/stable    
        """,
        ctx_obj=ctx_obj,
    )
    # - helm repo update
    # - helm search repo ${DEPLOY_HELM_CHART_NAME}

    # build PP_CONF section:
    # --set PP_CONF_1={PP_CONF_1} \
    # --set PP_CONF_2={PP_CONF_2} \
    # --set PP_CONF_3={PP_CONF_3} \
    pp_conf_n = "\n".join(
        [
            rf"--set PP_CONF_{i+1}={c} \ ".strip() if len(c) > 0 else r" \ "
            for i, c in enumerate(
                [
                    conf_file1 or "",
                    conf_file2 or "",
                    conf_file3 or "",
                    *conf_file,
                ]
            )
        ]
    )

    # helm_values_file section:
    #  -f values_overridden.yaml \
    #  -f values_overridden2.yaml \
    helm_values_file_n = "\n".join(
        [rf"-f {c} \ ".strip() for c in helm_values_file]
    )

    # helm_values section:
    #  --set GIT_CHECKOUT={GIT_CHECKOUT} \
    #  --set GIT_REPO={GIT_REPO} \
    #  --set GIT_BRANCH={GIT_BRANCH} \
    #  --set GIT_TOKEN={GIT_TOKEN} \
    helm_values_n = "\n".join(
        [rf"--set {c} \ ".strip() for c in helm_values]
    )

    # install the given chart version
    call(
        f"""
        helm upgrade \
          {deploy_helm_chart_name} \
          repo${deploy_gitlab_project_id}/{deploy_helm_chart_name} \
          --version {version} \
          --install \
          -n {deploy_k8s_namespace} \
          {pp_conf_n}
          {helm_values_file_n}
          {helm_values_n}
          --create-namespace
        """,
        ctx_obj=ctx_obj,
    )

    # Create a secret with the token credentials from gitlab
    # delete first to avoid an error
    call(
        f"""
        kubectl delete secret registry-credentials \
            -n {deploy_k8s_namespace} \
            --ignore-not-found
    """,
        ctx_obj=ctx_obj,
    )
    call(
        f"""
        kubectl create secret docker-registry registry-credentials  \
            -n {deploy_k8s_namespace} \
            --docker-server=https://registry.gitlab.com  \
            --docker-username=deploy-user \
            --docker-password={gitlab_token_registry_read} \
            --docker-email=support@paxpar.tech    
    """,
        ctx_obj=ctx_obj,
    )

    # Create a secret with the token credentials from gitea
    # delete first to avoid an error
    call(
        f"""
      kubectl delete secret gitea-registry-credentials \
        -n {deploy_k8s_namespace} \
        --ignore-not-found
    """,
        ctx_obj=ctx_obj,
    )
    call(
        f"""
      kubectl create secret docker-registry gitea-registry-credentials  \
        -n {deploy_k8s_namespace} \
        --docker-server=https://gitea.arundo.tech  \
        --docker-username=deploy-user \
        --docker-password={gitea_token_registry_read} \
        --docker-email=support@paxpar.tech    
    """,
        ctx_obj=ctx_obj,
    )

    # Create certmanager issuer
    ###TODO

    # some final info
    call(
        f"""kubectl get pods -n {deploy_k8s_namespace}""",
        ctx_obj=ctx_obj,
    )
    call(
        f"""helm list -n {deploy_k8s_namespace}""",
        ctx_obj=ctx_obj,
    )

    print(
        f"app {deploy_helm_chart_name} version {version} deployed on cluster/namespace {gitlab_k8s_agent}/{deploy_k8s_namespace}."
    )

    # backup image and upload to package registry
    # <!> microk8s n'est pas dans alpinelinux...
    # - microk8s ctr images export core_${DEPLOY_VERSION_RELEASE}.tar registry.gitlab.com/arundo-tech/paxpar/core:v${DEPLOY_VERSION_RELEASE}
    # - |
    #  curl \
    #    --header "JOB-TOKEN: $CI_JOB_TOKEN" \
    #    --upload-file ./core_${DEPLOY_VERSION_RELEASE}.tar \
    #    "${CI_API_V4_URL}/projects/${CI_PROJECT_ID}/packages/generic/api_package/${VERSION_RELEASE}/paxparcore.tar"
